<?php
/**
 * Jonah storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'charset'       The database's internal charset.</pre>
 *
 * Required by some database implementations:<pre>
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'database'      The name of the database.
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.</pre>
 *
 * The table structure can be created by the scripts/db/jonah_news.sql
 * script. The needed tables are jonah_channels and jonah_stories.
 *
 * $Horde: jonah/lib/News/sql.php,v 1.73 2007/03/12 05:02:40 mrubinsk Exp $
 *
 * Copyright 2002-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file LICENSE for license information (BSD). If you did not
 * did not receive this file, see http://cvs.horde.org/co.php/jonah/LICENSE.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jan Schneider <jan@horde.org>
 * @package Jonah
 */
class Jonah_News_sql extends Jonah_News {

    /**
     * Handle for the current database connection.
     *
     * @var DB
     */
    var $_db;

    /**
     * Boolean indicating whether or not we're connected to the SQL server.
     *
     * @var boolean
     */
    var $_connected = false;

    /**
     * Saves a channel to the backend.
     *
     * @param array $info  The channel to add.
     *                     Must contain a combination of the following
     *                     entries:
     * <pre>
     * 'channel_id'       If empty a new channel is being added, otherwise one
     *                    is being edited.
     * 'channel_name'     The headline.
     * 'channel_desc'     A description of this channel.
     * 'channel_type'     Whether internal or external.
     * 'channel_interval' If external then interval at which to refresh.
     * 'channel_link'     The link to the source.
     * 'channel_url'      The url from where to fetch the story list.
     * 'channel_image'    A channel image.
     * </pre>
     *
     * @return int|PEAR_Error  The channel ID on success, PEAR_Error on
     *                         failure.
     */
    function saveChannel(&$info)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        if (empty($info['channel_id'])) {
            $info['channel_id'] = $this->_db->nextId('jonah_channels');
            if (is_a($info['channel_id'], 'PEAR_Error')) {
                Horde::logMessage($info['channel_id'], __FILE__, __LINE__, PEAR_LOG_ERR);
                return $info['channel_id'];
            }
            $sql = 'INSERT INTO jonah_channels' .
                   ' (channel_id, channel_name, channel_type, channel_desc, channel_interval, channel_url, channel_link, channel_page_link, channel_story_url, channel_img)' .
                   ' VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)';
            $values = array();
        } else {
            $sql = 'UPDATE jonah_channels' .
                   ' SET channel_id = ?, channel_name = ?, channel_type = ?, channel_desc = ?, channel_interval = ?, channel_url = ?, channel_link = ?, channel_page_link = ?, channel_story_url = ?, channel_img = ?' .
                   ' WHERE channel_id = ?';
            $values = array((int)$info['channel_id']);
        }

        array_unshift($values,
                      (int)$info['channel_id'],
                      String::convertCharset($info['channel_name'], NLS::getCharset(), $this->_params['charset']),
                      (int)$info['channel_type'],
                      isset($info['channel_desc']) ? $info['channel_desc'] : null,
                      isset($info['channel_interval']) ? (int)$info['channel_interval'] : null,
                      isset($info['channel_url']) ? $info['channel_url'] : null,
                      isset($info['channel_link']) ? $info['channel_link'] : null,
                      isset($info['channel_page_link']) ? $info['channel_page_link'] : null,
                      isset($info['channel_story_url']) ? $info['channel_story_url'] : null,
                      isset($info['channel_img']) ? $info['channel_img'] : null);
        Horde::logMessage('SQL Query by Jonah_News_sql::saveChannel(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql, $values);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return $info['channel_id'];
    }

    /**
     * Get a list of stored channels.
     *
     * @param integer $type  The type of channel to filter for. Possible
     *                       values are either JONAH_INTERNAL_CHANNEL
     *                       to fetch only a list of internal channels,
     *                       or JONAH_EXTERNAL_CHANNEL for only external.
     *                       If null both channel types are returned.
     *
     * @return mixed         An array of channels or PEAR_Error on error.
     */
    function getChannels($type = null)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $wsql = '';
        if (!is_null($type)) {
            if (!is_array($type)) {
                $type = array($type);
            }
            $type = array_map(create_function('$a', 'return "channel_type = " . (int)$a;'), $type);
            $wsql = 'WHERE ' . implode(' OR ', $type);
        }

        $sql = sprintf('SELECT channel_id, channel_name, channel_type, channel_updated FROM jonah_channels %s ORDER BY channel_name', $wsql);

        Horde::logMessage('SQL Query by Jonah_News_sql::getChannels(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }
        for ($i = 0; $i < count($result); $i++) {
            $result[$i]['channel_name'] = String::convertCharset($result[$i]['channel_name'], $this->_params['charset']);
        }

        return $result;
    }

    /**
     */
    function _getChannel($channel_id)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'SELECT * FROM jonah_channels WHERE channel_id = ' . (int)$channel_id;

        Horde::logMessage('SQL Query by Jonah_News_sql::_getChannel(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        } elseif (empty($result)) {
            return PEAR::raiseError(sprintf(_("Channel id \"%s\" not found."), $channel_id));
        }

        $result['channel_name'] = String::convertCharset($result['channel_name'], $this->_params['charset']);
        if ($result['channel_type'] == JONAH_COMPOSITE_CHANNEL) {
            $channels = explode(':', $result['channel_url']);
            if (count($channels)) {
                $sql = 'SELECT MAX(channel_updated) FROM jonah_channels WHERE channel_id IN (' . implode(',', $channels) . ')';
                Horde::logMessage('SQL Query by Jonah_News_sql::_getChannel(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
                $updated = $this->_db->getOne($sql);
                if (is_a($updated, 'PEAR_Error')) {
                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                } else {
                    $result['channel_updated'] = $updated;
                    $this->_timestampChannel($channel_id, $updated);
                }
            }
        }

        return $result;
    }

    /**
     */
    function _timestampChannel($channel_id, $timestamp)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = sprintf('UPDATE jonah_channels SET channel_updated = %s WHERE channel_id = %s',
                       (int)$timestamp,
                       (int)$channel_id);
        Horde::logMessage('SQL Query by Jonah_News_sql::_timestampChannel(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }
        return $result;
    }

    /**
     */
    function _readStory($story_id)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'UPDATE jonah_stories SET story_read = story_read + 1 WHERE story_id = ' . (int)$story_id;
        Horde::logMessage('SQL Query by Jonah_News_sql::_readStory(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }
        return $result;
    }

    /**
     */
    function _deleteChannel($channel_id)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'DELETE FROM jonah_channels WHERE channel_id = ?';
        $values = array($channel_id);

        Horde::logMessage('SQL Query by Jonah_News_sql::deleteChannel(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql, $values);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }

        return $result;
    }

    /**
     * @param array &$info
     */
    function _saveStory(&$info)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        if (empty($info['story_id'])) {
            $info['story_id'] = $this->_db->nextId('jonah_stories');
            if (is_a($info['story_id'], 'PEAR_Error')) {
                Horde::logMessage($info['story_id'], __FILE__, __LINE__, PEAR_LOG_ERR);
                return $info['story_id'];
            }
            $channel = $this->getChannel($info['channel_id']);
            $permalink = $this->getStoryLink($channel, $info);
            $sql = 'INSERT INTO jonah_stories (story_id, channel_id, story_title, story_desc, story_body_type, story_body, story_url, story_published, story_updated, story_read, story_permalink) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)';
            $values = array($permalink);
        } else {
            $sql = 'UPDATE jonah_stories SET story_id = ?, channel_id = ?, story_title = ?, story_desc = ?, story_body_type = ?, story_body = ?, story_url = ?, story_published = ?, story_updated = ?, story_read = ? WHERE story_id = ?';
            $values = array((int)$info['story_id']);
        }

        if (empty($info['story_read'])) {
            $info['story_read'] = 0;
        }

        array_unshift($values,
                      (int)$info['story_id'],
                      (int)$info['channel_id'],
                      String::convertCharset($info['story_title'], NLS::getCharset(), $this->_params['charset']),
                      String::convertCharset($info['story_desc'], NLS::getCharset(), $this->_params['charset']),
                      $info['story_body_type'],
                      isset($info['story_body']) ? String::convertCharset($info['story_body'], NLS::getCharset(), $this->_params['charset']) : null,
                      isset($info['story_url']) ? $info['story_url'] : null,
                      isset($info['story_published']) ? (int)$info['story_published'] : null,
                      time(),
                      (int)$info['story_read']);

        Horde::logMessage('SQL Query by Jonah_News_sql::_saveStory(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql, $values);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        $this->_timestampChannel($info['channel_id'], time());
        return true;
    }

    /**
     * Converts the text fields of a story from the backend charset to the
     * output charset.
     *
     * @param array $story  A story hash.
     *
     * @return array  The converted hash.
     */
    function _convertFromBackend($story)
    {
        $story['story_title'] = String::convertCharset($story['story_title'], $this->_params['charset'], NLS::getCharset());
        $story['story_desc'] = String::convertCharset($story['story_desc'], $this->_params['charset'], NLS::getCharset());
        if (isset($story['story_body'])) {
            $story['story_body'] = String::convertCharset($story['story_body'], $this->_params['charset'], NLS::getCharset());
        }
        return $story;
    }

    /**
     * Returns the most recent or all stories from a channel.
     *
     * @param integer $channel_id  The news channel to get stories from.
     * @param integer $max         The maximum number of stories to get.
     * @param integer $from        The number of the story to start with.
     * @param integer $date        The timestamp of the date to start with.
     * @param boolean $unreleased  Whether to return not yet released stories.
     *
     * @return array  The specified number (or less, if there are fewer) of
     *                stories from the given channel.
     */
    function _getStories($channel_id, $max, $from = 0, $date = null,
                         $unreleased = false, $order)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'SELECT * FROM jonah_stories WHERE channel_id = ?';
        $values = array((int)$channel_id);

        if ($unreleased) {
            if ($date !== null) {
                $sql .= ' AND story_published <= ?';
                $values[] = $date;
            }
        } else {
            if ($date === null) {
                $date = time();
            } else {
                $date = max($date, time());
            }
            $sql .= ' AND story_published <= ?';
            $values[] = $date;
        }

        switch ($order) {
        case JONAH_ORDER_UPDATED:
            $sql .= ' ORDER BY story_updated DESC';
            break;
        case JONAH_ORDER_READ:
            $sql .= ' ORDER BY story_read DESC';
            break;
        case JONAH_ORDER_COMMENTS:
            //@TODO
            break;
        }

        if (!is_null($max)) {
            $sql = $this->_db->modifyLimitQuery($sql, (int)$from, (int)$max, $values);
        }

        Horde::logMessage('SQL Query by Jonah_News_sql::_getStories(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getAll($sql, $values, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
        for ($i = 0; $i < count($result); $i++) {
            $result[$i] = $this->_convertFromBackend($result[$i]);
            if (empty($result[$i]['story_permalink'])) {
                $this->_addPermalink($result[$i]);
            }
        }
        return $result;
    }

    /**
     */
    function _getStory($story_id, $read = false)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'SELECT * FROM jonah_stories WHERE story_id = ?';
        $values = array((int)$story_id);

        Horde::logMessage('SQL Query by Jonah_News_sql::_getStory(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getRow($sql, $values, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        } elseif (empty($result)) {
            return PEAR::raiseError(sprintf(_("Story id \"%s\" not found."), $story_id));
        }
        $result = $this->_convertFromBackend($result);
        if (empty($result['story_permalink'])) {
            $this->_addPermalink($result);
        }
        if ($read) {
            $this->_readStory($story_id);
        }

        return $result;
    }

    /**
     */
    function _getStoryByUrl($channel_id, $story_url)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'SELECT * FROM jonah_stories' .
               ' WHERE channel_id = ? AND story_url = ?';
        $values = array((int)$channel_id, $story_url);

        Horde::logMessage('SQL Query by Jonah_News_sql::_getStoryByUrl(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getRow($sql, $values, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        } elseif (empty($result)) {
            return PEAR::raiseError(sprintf(_("Story URL \"%s\" not found."), $story_url));
        }
        $result = $this->_convertFromBackend($result);
        if (empty($result['story_permalink'])) {
            $this->_addPermalink($result);
        }

        return $result;
    }

    /**
     * Adds a missing permalink to a story.
     *
     * @param array $story  A story hash.
     */
    function _addPermalink(&$story)
    {
        $channel = $this->getChannel($story['channel_id']);
        if (is_a($channel, 'PEAR_Error')) {
            return;
        }
        $sql = 'UPDATE jonah_stories SET story_permalink = ? WHERE story_id = ?';
        $values = array($this->getStoryLink($channel, $story), $story['story_id']);
        Horde::logMessage('SQL Query by Jonah_News_sql::_addPermalink(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql, $values);
        if (!is_a($result, 'PEAR_Error')) {
            $story['story_permalink'] = $values[0];
        }
    }

    /**
     * Gets the latest released story from a given internal channel
     *
     * @param int $channel_id  The channel id.
     *
     * @return int  The story id.
     */
    function getLatestStoryId($channel_id)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'SELECT story_id FROM jonah_stories' .
               ' WHERE channel_id = ? AND story_published <= ?' .
               ' ORDER BY story_updated DESC';
        $values = array((int)$channel_id, time());

        Horde::logMessage('SQL Query by Jonah_News_sql::getLatestStoryId(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getRow($sql, $values, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        } elseif (empty($result)) {
            return PEAR::raiseError(sprintf(_("Channel \"%s\" not found."), $channel_id));
        }

        return $result['story_id'];
    }

    /**
     */
    function deleteStory($channel_id, $story_id)
    {
        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
            return $result;
        }

        $sql = 'DELETE FROM jonah_stories' .
               ' WHERE channel_id = ? AND story_id = ?';
        $values = array((int)$channel_id, (int)$story_id);

        Horde::logMessage('SQL Query by Jonah_News_sql::deleteStory(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql, $values);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result->getMessage(), __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return true;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean    True on success; PEAR_Error on failure.
     */
    function _connect()
    {
        if ($this->_connected) {
            return true;
        }

        Horde::assertDriverConfig($this->_params, 'news',
            array('phptype', 'charset'),
            'jonah news SQL');

        if (!isset($this->_params['database'])) {
            $this->_params['database'] = '';
        }
        if (!isset($this->_params['username'])) {
            $this->_params['username'] = '';
        }
        if (!isset($this->_params['hostspec'])) {
            $this->_params['hostspec'] = '';
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            return $this->_db;
        }

        // Set DB portability options.
        switch ($this->_db->phptype) {
        case 'mssql':
            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
            break;
        default:
            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
        }

        $this->_connected = true;
        return true;
    }

}
